def chromosome_to_cycle(Chromosome):
	Nodes = [0 for i in range(len(Chromosome) * 2)]
	for j in range(len(Chromosome)):
		i = Chromosome[j]
		if i > 0:
			Nodes[2 * j] = 2 * i - 1
			Nodes[2 * j + 1] = 2 * i
		else:
			Nodes[2 * j] = - 2 * i
			Nodes[2 * j + 1] = - 2 * i - 1
	return Nodes

def cycle_to_chromosome(Nodes):
	Chromosome = [0 for i in range(len(Nodes) // 2)]
	for i in range(len(Nodes) // 2):
		if Nodes[2 * i] < Nodes[2 * i + 1]:
			Chromosome[i] = Nodes[2 * i + 1] // 2
		else:
			Chromosome[i] = - Nodes[2 * i] // 2
	return Chromosome

def colored_edges(P):
	Edges = []
	for Chromosome in P:
		Nodes = chromosome_to_cycle(Chromosome)
		for j in range(len(Chromosome) - 1):
			Edges.append((Nodes[2 * j + 1], Nodes[2 * j + 2]))
		Edges.append((Nodes[-1], Nodes[0])) # Jer je ciklicna struktura
	return Edges

def black_edges(P):
	Edges = []
	for Chromosome in P:
		Nodes = chromosome_to_cycle(Chromosome)
		for j in range(len(Chromosome)):
			Edges.append((Nodes[2 * j], Nodes[2 * j + 1]))
	return Edges	

def edges_to_graph(GraphEdges):
	Graph = {}
	for edge in GraphEdges:
		if edge[0] not in Graph:
			Graph[edge[0]] = []
		Graph[edge[0]].append(edge[1])
	return Graph

def dfs(Graph, v, marked):
	marked[v] = True
	path = [v]

	while len(path) > 0:
		top = path[-1]

		has_neighbor = False

		for w in Graph[top]:
			if w == v:
				return path

			if w not in marked:
				marked[w] = True
				has_neighbor = True
				path.append(w)
				break

		if not has_neighbor:
			path.pop()

	return None

# Pronalazi sve cikluse u grafu
def cycle_nodes(GenomeGraph):
	Cycles = []
	marked = {}

	for v in GenomeGraph:
		if v not in marked:
			path = dfs(GenomeGraph, v, marked)

			if path != None:
				if abs(path[1] - path[0]) > 1:
					tail = [path[-1]]
					del path[-1]
					path = tail + path
				Cycles.append(path)
	return Cycles

	

def graph_to_genome(GenomeGraph):
 	P = []
 	for cycle in cycle_nodes(GenomeGraph):
 		P.append(cycle_to_chromosome(cycle))
 	return P

def remove_edge_from_graph(Graph, edge):
	(v, w) = edge

	Graph[v].remove(w)

	return Graph

def add_edge_to_graph(Graph, edge):
	(v, w) = edge

	if v not in Graph:
		Graph[v] = []

	Graph[v].append(w)

	return Graph

def two_break_on_genome_graph(GenomeGraph, i, ip, j, jp):
	remove_edge_from_graph(GenomeGraph, (i, ip))
	remove_edge_from_graph(GenomeGraph, (j, jp))
	add_edge_to_graph(GenomeGraph, (i, jp))
	add_edge_to_graph(GenomeGraph, (j, ip))
	return GenomeGraph

def two_break_on_genome(P, i, ip, j, jp):
	GenomeGraph = edges_to_graph(black_edges(P) + colored_edges(P))
	GenomeGraph = two_break_on_genome_graph(GenomeGraph, i, ip, j, jp)
	P = graph_to_genome(GenomeGraph)
	return P


def main():
	ChromosomeStart = [1, -2, -3, 4]
	Nodes = chromosome_to_cycle(ChromosomeStart)
	# print(Nodes)
	Chromosome = cycle_to_chromosome(Nodes)
	# print(Chromosome)
	# print(colored_edges([Chromosome]))
	# print(black_edges([Chromosome]))

	G = edges_to_graph(colored_edges([Chromosome]) + black_edges([Chromosome]))

	# G = two_break_on_genome_graph(G, 3, 6, 8, 1)
	# print(G)

	print(two_break_on_genome([Chromosome], 3, 6, 8, 1))

	# print(graph_to_genome(G))

	# remove_edge_from_graph(G, (1,2))
	# add_edge_to_graph(G, (1,2))
	# print(G)



if __name__ == "__main__":
	main()
